@@ -1,9 +1,12 @@
 import asyncio
 import logging
 import os
+import signal
+import sys
 from concurrent.futures import Executor
 from typing import (
-    Any, Callable, Coroutine, Iterable, MutableSet, Optional, TypeVar, Union,
+    Any, Callable, Coroutine, FrozenSet, MutableSet, Optional, Set, Tuple,
+    TypeVar, Union,
 )
 from weakref import WeakSet
 
@@ -40,6 +43,24 @@ def _get_env_convert(name: str, converter: Callable[..., T], default: T) -> T:
     return converter(value)
 
 
+class OSSignalHandler:
+    def __init__(
+        self, sig: int, handler: Callable[[int], None],
+    ):
+        self.default_handler = signal.getsignal(sig)
+        self.signal = sig
+        self.handler = handler
+
+    def __callback(self, *_: Any) -> None:
+        self.handler(self.signal)
+
+    def apply(self) -> None:
+        signal.signal(self.signal, self.__callback)
+
+    def restore(self) -> None:
+        signal.signal(self.signal, self.default_handler)
+
+
 @final
 class Entrypoint:
     DEFAULT_LOG_LEVEL: str = os.getenv(
@@ -49,7 +70,7 @@ class Entrypoint:
         "AIOMISC_LOG_FORMAT", LogFormat.default(),
     )
     DEFAULT_LOG_DATE_FORMAT: Optional[str] = os.getenv(
-        "AIOMISC_LOG_DATE_FORMAT"
+        "AIOMISC_LOG_DATE_FORMAT",
     )
 
     DEFAULT_AIOMISC_DEBUG: bool = _get_env_bool("AIOMISC_DEBUG", "0")
@@ -68,6 +89,9 @@ class Entrypoint:
     DEFAULT_AIOMISC_POOL_SIZE: Optional[int] = _get_env_convert(
         "AIOMISC_POOL_SIZE", int, None,
     )
+    AIOMISC_SHUTDOWN_TIMEOUT: float = _get_env_convert(
+        "AIOMISC_SHUTDOWN_TIMEOUT", float, 60.,
+    )
 
     PRE_START = Signal()
     POST_STOP = Signal()
@@ -93,21 +117,23 @@ class Entrypoint:
         CURRENT_ENTRYPOINT.set(self)
         EVENT_LOOP.set(self.loop)
 
-        for signal in (
-            self.pre_start, self.post_stop,
-            self.pre_stop, self.post_start,
-        ):
-            signal.freeze()
+        signals = (
+            self.pre_start, self.post_stop, self.pre_stop, self.post_start,
+        )
 
-        services = self.services
-        await self.pre_start.call(entrypoint=self, services=services)
-        await asyncio.gather(*[self._start_service(svc) for svc in services])
-        await self.post_start.call(entrypoint=self, services=services)
+        for sig in signals:
+            sig.freeze()
+
+        await self.start_services(*self.__passed_services)
+        del self.__passed_services
+
+        for handler in self._signal_handlers:
+            handler.apply()
 
     def __init__(
         self, *services: Service,
         loop: Optional[asyncio.AbstractEventLoop] = None,
-        pool_size: Optional[int] = None,
+        pool_size: Optional[int] = DEFAULT_AIOMISC_POOL_SIZE,
         log_level: Union[int, str] = DEFAULT_LOG_LEVEL,
         log_format: Union[str, LogFormat] = DEFAULT_LOG_FORMAT,
         log_buffering: bool = DEFAULT_AIOMISC_BUFFERING,
@@ -117,6 +143,8 @@ class Entrypoint:
         log_config: bool = DEFAULT_AIOMISC_LOG_CONFIG,
         policy: asyncio.AbstractEventLoopPolicy = event_loop_policy,
         debug: bool = DEFAULT_AIOMISC_DEBUG,
+        catch_signals: Tuple[int, ...] = (signal.SIGINT, signal.SIGTERM),
+        shutdown_timeout: Union[int, float] = AIOMISC_SHUTDOWN_TIMEOUT,
     ):
 
         """ Creates a new Entrypoint
@@ -130,8 +158,14 @@ class Entrypoint:
         :param log_buffer_size: Buffer size for logging
         :param log_flush_interval: interval in seconds for flushing logs
         :param log_config: if False do not configure logging
+        :param catch_signals: Perform shutdown when this signals will
+                              be received
+        :param shutdown_timeout: Timeout in seconds for graceful shutdown
         """
 
+        self.__passed_services: FrozenSet[Service] = frozenset(services)
+
+        self._services: Set[Service] = set()
         self._debug = debug
         self._loop = loop
         self._loop_owner = False
@@ -139,6 +173,13 @@ class Entrypoint:
         self._thread_pool: Optional[ExecutorType] = None
         self._closing: Optional[asyncio.Event] = None
 
+        self._signal_handlers = [
+            OSSignalHandler(sig, self._on_interrupt_callback)
+            for sig in catch_signals
+        ]
+
+        self.catch_signals = catch_signals
+        self.shutdown_timeout = float(shutdown_timeout)
         self.ctx: Optional[Context] = None
         self.log_buffer_size = log_buffer_size
         self.log_buffering = log_buffering
@@ -148,20 +189,19 @@ class Entrypoint:
         self.log_format = log_format
         self.log_level = log_level
         self.policy = policy
+
+        # signals
         self.pool_size = pool_size
-        self.__services = set(services)
-        self.shutting_down = False
         self.pre_start = self.PRE_START.copy()
         self.post_start = self.POST_START.copy()
         self.pre_stop = self.PRE_STOP.copy()
         self.post_stop = self.POST_STOP.copy()
-        self._lock = asyncio.Lock()
 
         if self.log_config:
             aiomisc_log.basic_config(
                 level=self.log_level,
                 log_format=self.log_format,
-                date_format=log_date_format
+                date_format=log_date_format,
             )
 
         if self._loop is not None:
@@ -170,8 +210,8 @@ class Entrypoint:
         CURRENT_ENTRYPOINT.set(self)
 
     @property
-    def services(self) -> Iterable[Service]:
-        return tuple(self.__services)
+    def services(self) -> Tuple[Service, ...]:
+        return tuple(self._services)
 
     async def closing(self) -> None:
         # Lazy initialization because event loop might be not exists
@@ -206,7 +246,8 @@ class Entrypoint:
     def __exit__(
         self, exc_type: Any, exc_val: Any, exc_tb: Any,
     ) -> None:
-        if self.loop.is_closed():
+        loop = self.loop
+        if loop.is_closed():
             return
 
         if self.log_config:
@@ -217,12 +258,9 @@ class Entrypoint:
                 buffered=False,
             )
 
-        self.loop.run_until_complete(
-            self.__aexit__(exc_type, exc_val, exc_tb),
-        )
-
+        loop.run_until_complete(self.__aexit__(exc_type, exc_val, exc_tb))
         if self._loop_owner and self._loop is not None:
-            self._loop.close()
+            loop.close()
 
     async def __aenter__(self) -> "Entrypoint":
         if self._loop is None:
@@ -236,20 +274,37 @@ class Entrypoint:
     async def __aexit__(
         self, exc_type: Any, exc_val: Any, exc_tb: Any,
     ) -> None:
-        await self.pre_stop.call(entrypoint=self)
+        await self._stop(exc_val)
+
+    if sys.version_info < (3, 9):
+        async def __shutdown_thread_pool(
+            self, loop: asyncio.AbstractEventLoop,
+        ) -> None:
+            result = self._thread_pool.shutdown()
+            if hasattr(result, "__await__"):
+                await result
+    else:
+        def __shutdown_thread_pool(
+            self, loop: asyncio.AbstractEventLoop,
+        ) -> Coroutine[Any, Any, None]:
+            return loop.shutdown_default_executor()
+
+    async def _stop(self, exc: Exception) -> None:
+        loop = self.loop
+
+        for handler in self._signal_handlers:
+            handler.restore()
 
         try:
-            if self.loop.is_closed():
+            if loop.is_closed():
                 return
-
-            await self.graceful_shutdown(exc_val)
-            self.shutting_down = True
+            await self.graceful_shutdown(exc)
         finally:
             if self.ctx:
                 self.ctx.close()
 
             if self._thread_pool:
-                self._thread_pool.shutdown()
+                await self.__shutdown_thread_pool(loop)
 
     async def _start_service(
         self, svc: Service,
@@ -262,6 +317,8 @@ class Entrypoint:
             ),
         )
 
+        self._services.add(svc)
+
         await asyncio.wait(
             (start_task, ev_task),
             return_when=asyncio.FIRST_COMPLETED,
@@ -285,19 +342,36 @@ class Entrypoint:
             await asyncio.gather(*[self._start_service(s) for s in svc])
         finally:
             await self.post_start.call(entrypoint=self, services=svc)
-            for s in svc:
-                self.__services.add(s)
 
     async def stop_services(
         self, *svc: Service, exc: Optional[Exception] = None,
     ) -> None:
         await self.pre_stop.call(entrypoint=self, services=svc)
 
+        if not svc:
+            await self.post_stop.call(entrypoint=self, services=svc)
+            return
+
+        tasks = []
+
         try:
-            await asyncio.gather(*[s.stop(exc) for s in svc])
-        finally:
             for s in svc:
-                self.__services.remove(s)
+                try:
+                    log.debug("Stopping service %r", s)
+                    coro = s.stop(exc)
+                    if hasattr(coro, "__await__"):
+                        tasks.append(self.loop.create_task(coro))
+                except TypeError as e:
+                    log.warning("Failed to stop service %r:\n%r", svc, e)
+                    log.debug("Service stop failed traceback", exc_info=True)
+
+            await asyncio.gather(*tasks)
+        finally:
+            await cancel_tasks(tasks)
+
+            for s in svc:
+                self._services.discard(s)
+
             await self.post_stop.call(entrypoint=self, services=svc)
 
     async def _cancel_background_tasks(self) -> None:
@@ -309,29 +383,38 @@ class Entrypoint:
         if self._closing:
             self._closing.set()
 
-        tasks = []
-        for svc in self.__services:
-            try:
-                coro = svc.stop(exception)
-            except TypeError as e:
-                log.warning(
-                    "Failed to stop service %r:\n%r", svc, e,
-                )
-                log.debug("Service stop failed traceback", exc_info=True)
-            else:
-                tasks.append(asyncio.shield(coro))
-
-        if tasks:
-            await asyncio.gather(*tasks, return_exceptions=True)
-
         await cancel_tasks(set(self._tasks))
-
-        await self.post_stop.call(entrypoint=self)
+        await self.stop_services(*self._services, exc=exception)
 
         if self._loop_owner:
             await self._cancel_background_tasks()
+
         await self.loop.shutdown_asyncgens()
 
+    def _on_interrupt_callback(self, _: Any) -> None:
+        loop = self.loop
+        self.loop.call_soon_threadsafe(self._on_interrupt, loop)
+
+    def _on_interrupt(self, loop: asyncio.AbstractEventLoop) -> None:
+        async def shutdown() -> None:
+            log.warning("Interrupt signal received, shutting down...")
+            try:
+                await asyncio.wait_for(
+                    self._stop(RuntimeError("Interrupt signal received")),
+                    timeout=self.shutdown_timeout,
+                )
+            except asyncio.TimeoutError as e:
+                log.warning(
+                    "Shutdown did not happen in %s seconds, aborting.",
+                    self.shutdown_timeout,
+                )
+                # 70 from sysexits.h means "internal software error"
+                raise SystemExit(70) from e
+
+        self.loop.create_task(shutdown()).add_done_callback(
+            lambda _: loop.stop(),
+        )
+
 
 CURRENT_ENTRYPOINT: StrictContextVar[Entrypoint] = StrictContextVar(
     "CURRENT_ENTRYPOINT",
