@@ -46,10 +46,11 @@ class TLSServer(SimpleServer):
     PROTO_NAME = "tls"
 
     def __init__(
-        self, *, address: str = None, port: int = None,
-        cert: PathOrStr, key: PathOrStr, ca: PathOrStr = None,
+        self, *, address: Optional[str] = None, port: Optional[int] = None,
+        cert: PathOrStr, key: PathOrStr, ca: Optional[PathOrStr] = None,
         require_client_cert: bool = False, verify: bool = True,
-        options: OptionsType = (), sock: socket.socket = None, **kwargs: Any
+        options: OptionsType = (), sock: Optional[socket.socket] = None,
+        **kwargs: Any,
     ):
 
         self.__ssl_options = (
@@ -99,7 +100,7 @@ class TLSServer(SimpleServer):
 
     async def start(self) -> None:
         ssl_context = await self.loop.run_in_executor(
-            None, get_ssl_context, *self.__ssl_options
+            None, get_ssl_context, *self.__ssl_options,
         )
 
         self.socket = self.make_socket()
@@ -110,7 +111,7 @@ class TLSServer(SimpleServer):
             ssl=ssl_context,
         )
 
-    async def stop(self, exc: Exception = None) -> None:
+    async def stop(self, exc: Optional[Exception] = None) -> None:
         await super().stop(exc)
 
         if self.server:
@@ -144,8 +145,8 @@ class TLSClient(TCPClient, ABC):
     def __init__(
         self, address: str, port: int, *,
         cert: PathOrStr, key: PathOrStr,
-        ca: PathOrStr = None, verify: bool = True,
-        **kwargs: Any
+        ca: Optional[PathOrStr] = None, verify: bool = True,
+        **kwargs: Any,
     ) -> None:
         self.__ssl_options = (
             cert, key, ca, verify, False, ssl.Purpose.SERVER_AUTH,
@@ -186,8 +187,8 @@ class RobustTLSClient(RobustTCPClient, ABC):
     def __init__(
         self, address: str, port: int, *,
         cert: PathOrStr, key: PathOrStr,
-        ca: PathOrStr = None, verify: bool = True,
-        **kwargs: Any
+        ca: Optional[PathOrStr] = None, verify: bool = True,
+        **kwargs: Any,
     ):
         self.__ssl_options = (
             cert, key, ca, verify, False, ssl.Purpose.SERVER_AUTH,
