@@ -15,7 +15,7 @@ from typing import (
     Set, Tuple, TypeVar,
 )
 
-from .compat import get_current_loop
+from ._context_vars import EVENT_LOOP
 from .counters import Statistic
 from .iterator_wrapper import IteratorWrapper
 
@@ -398,7 +398,7 @@ class CoroutineWaiter:
         loop: Optional[asyncio.AbstractEventLoop] = None,
     ):
         self.__coro: Coroutine[Any, Any, T] = coroutine
-        self.__loop = loop or get_current_loop()
+        self.__loop = loop or EVENT_LOOP.get()
         self.__event = threading.Event()
         self.__result: Optional[T] = None
         self.__exception: Optional[BaseException] = None
